print("import: 'reportlab'")
import reportlab

print("import: 'reportlab.graphics.charts'")
import reportlab.graphics.charts

print("import: 'reportlab.graphics.samples'")
import reportlab.graphics.samples

print("import: 'reportlab.graphics.widgets'")
import reportlab.graphics.widgets

print("import: 'reportlab.graphics.barcode'")
import reportlab.graphics.barcode

print("import: 'reportlab.graphics'")
import reportlab.graphics

print("import: 'reportlab.lib'")
import reportlab.lib

print("import: 'reportlab.pdfbase'")
import reportlab.pdfbase

print("import: 'reportlab.pdfgen'")
import reportlab.pdfgen

print("import: 'reportlab.platypus'")
import reportlab.platypus

